<?php

namespace Drupal\symfony_mailer\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\mailer_policy\PolicyHelperInterface;
use Drupal\symfony_mailer\AutowireTrait;
use Drupal\symfony_mailer\Plugin\Mailer\TestMailerInterface;

/**
 * Symfony Mailer test email form.
 */
class TestEmailForm extends FormBase {

  use AutowireTrait;

  /**
   * Constructs a new TestForm.
   *
   * @param \Drupal\symfony_mailer\Plugin\Mailer\TestMailerInterface $mailer
   *   The test mailer.
   * @param \Drupal\mailer_policy\PolicyHelperInterface $helper
   *   The policy helper.
   *
   * @internal
   */
  public function __construct(
    protected readonly TestMailerInterface $mailer,
    protected readonly ?PolicyHelperInterface $helper = NULL,
  ) {}

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'symfony_mailer_test_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['#tree'] = TRUE;

    $form['recipient'] = [
      '#title' => $this->t('Recipient'),
      '#type' => 'textfield',
      '#default_value' => '',
      '#description' => $this->t('Recipient email address. Leave blank to send to yourself.'),
    ];

    if ($this->helper) {
      $form['mailer_policy'] = $this->helper->renderTypePolicy('symfony_mailer');
    }

    $form['actions']['#type'] = 'actions';
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Send'),
      '#button_type' => 'primary',
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $to = $form_state->getValue('recipient') ?: $this->currentUser();
    $this->mailer->verify($to);
    $message = is_object($to) ?
      $this->t('An attempt has been made to send an email to you.') :
      $this->t('An attempt has been made to send an email to @to.', ['@to' => $to]);
    $this->messenger()->addMessage($message);
  }

}
